/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __FIFORECEIVEQUEUE_H__
#define __FIFORECEIVEQUEUE_H__





/***********************************************************************************/
/*****************************   FIFO_RECEIVEQUEUE_NODE   **************************/
/***********************************************************************************/

typedef class FIFO_ReceiveQueueNode *ptFIFO_ReceiveQueueNode;
class FIFO_ReceiveQueueNode
{
  private :
   ProcessID_type contents;                                  /* Contents of a node */
   ptFIFO_ReceiveQueueNode next;                                      /* Next node */

  public :
   FIFO_ReceiveQueueNode();
   FIFO_ReceiveQueueNode(ProcessID_type Process_ID);
   ~FIFO_ReceiveQueueNode();
   void setContents(ProcessID_type Process_ID);
   void setNext(ptFIFO_ReceiveQueueNode nextNode);
   ProcessID_type getContents();
   ptFIFO_ReceiveQueueNode getNext();
};



/**********************************************************************************/
/******************************   FIFO_RECEIVEQUEUE   *****************************/
/**********************************************************************************/

typedef class FIFO_ReceiveQueue *ptFIFO_ReceiveQueue;
class FIFO_ReceiveQueue
{
  private :
   ptFIFO_ReceiveQueueNode first;                      /* First node of the QUEUE */
   ptFIFO_ReceiveQueueNode last;                        /* Last node of the QUEUE */
   int max_size;                                       /* Size limit of the QUEUE */
   int current_size;                      /* Current number of nodes in the QUEUE */

  void setFirst(ptFIFO_ReceiveQueueNode Node);
  void setLast(ptFIFO_ReceiveQueueNode Node);
  ptFIFO_ReceiveQueueNode getFirst();
  ptFIFO_ReceiveQueueNode getLast();

 public :
  FIFO_ReceiveQueue();
  FIFO_ReceiveQueue(int size);
  ~FIFO_ReceiveQueue();
  int getMax_size();
  int getCurrent_size();

  void insert(ProcessID_type Process_ID);
  /* Insert Process_ID in the QUEUE */

  void remove(ProcessID_type& Process_ID);
  /* Remove the oldest node from the QUEUE and Process_ID is this node */

  bool empty();
  /* The QUEUE is empty */

  bool full();
  /* The QUEUE size limit is reached */

  void printFile(char *result_file);
  /* prints the QUEUE in result_file */

};



#endif // __FIFORECEIVEQUEUE_H__










