
module Sigali_CTL =
    pragmas 
      Comment "SIGNAL/Sigali-CTL INTERFACE."
      AUTHOR  "Loc Besnard/Julio Peralta (June 2007)"
    end pragmas

     process EX =
        ( ? setOfStates P;
          ! setOfStates RESULT;
        )
      pragmas 
         Sigali "&o1 :&n(&i1);"
         Comment "CTL Logic: (EXISTS NEXT) There exists a successor such that P holds."
      end pragmas;
      
     process AX =
        ( ? setOfStates P;
          ! setOfStates RESULT;
        )
      pragmas 
         Sigali "&o1 :&n(&i1);"
         Comment "CTL Logic: (ALWAYS NEXT) All the  successors satisfy P."
      end pragmas;
      
     process EU =
        ( ? setOfStates P1, P2;
          ! setOfStates RESULT;
        )
      pragmas 
        Sigali "&o1 :&n(&i1, &i2);"
        Comment "CTL Logic: (EXIST UNTIL) It exists a path on which P1 is true until P2 becomes true"
      end pragmas;
      
     process AU =
        ( ? setOfStates P1, P2;
          ! setOfStates RESULT;
        )
      pragmas 
        Sigali "&o1 : &n(&i1, &i2);"
        Comment "CTL Logic: (ALWAYS UNTIL) For all possible paths,  P1 is true until P2 becomes true"
      end pragmas;
      
      
     process EG =
        ( ? setOfStates P;
          ! setOfStates RESULT;
        )
      pragmas 
         Sigali "&o1 :&n(&i1);"
         Comment "CTL Logic: (ALWAYS in one path) there is always a path such that P is true"
      end pragmas;
      
     process AG =
        ( ? setOfStates P;
          ! setOfStates RESULT;
        )
      pragmas 
        Sigali "&o1 : &n(&i1);"
        Comment "CTL Logic: (ALWAYS in all paths) whatever the behavior of a system (S), P is true. It is applied on the reachable states"
      end pragmas;
      
     process EF =
        ( ? setOfStates P;
          ! setOfStates RESULT;
        )
      pragmas 
         Sigali "&o1 : &n(&i1);"
         Comment "CTL Logic: (EVENTUALLY in one path) There exists a finite path to a state where P is true"
      end pragmas;
      
     process AF =
       ( ? setOfStates P;
         ! setOfStates RESULT;
           )
       pragmas 
         Sigali "&o1 : &n(&i1);"
         Comment "CTL Logic: (EVENTUALLY for all paths) whatever the behavior of S, P will eventually become true in a finite number of transitions"
       end pragmas;
       
     process CHECK_CTL_FORMULAE =
       ( ? setOfStates P;
         ! boolean RESULT;
           )
       pragmas 
         Sigali "&o1 : &n(&i1);"
         Comment "CTL Logic: (VERIFICATION) True if the CTL formula, P, holds from/for the initial states of the system, false otherwise."
         "It is equivalent to subset(initial(S), P)"
       end pragmas;
       

      end %Sigali%; 
